/*
 * Copyright (c) 2000-2012 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * This file is part of nfc-manager
 *
 * PROPRIETARY/CONFIDENTIAL
 *
 * This software is the confidential and proprietary information of
 * SAMSUNG ELECTRONICS ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with SAMSUNG ELECTRONICS.
 *
 * SAMSUNG make no representations or warranties about the suitability
 * of the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a particular
 * purpose, or non-infringement. SAMSUNG shall not be liable for any
 * damages suffered by licensee as a result of using, modifying or
 * distributing this software or its derivatives.
 *
 */

#ifndef __NET_NFC_NDEF_RECORD_H__
#define __NET_NFC_NDEF_RECORD_H__

#include "net_nfc_typedef.h"

#ifdef __cplusplus
  extern "C" {
#endif

/**
	This function create wifi configure handler instance.

	@param[out] 	config 		instance handler
	@param[in] 	type 				Carrier types it would be wifi add hoc or wifi AP

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_OUT_OF_BOUND			type value is not enum value
	@exception NET_NFC_ALLOC_FAIL			allocation is failed
*/

net_nfc_error_e net_nfc_create_carrier_config (net_nfc_carrier_config_h * config, net_nfc_conn_handover_carrier_type_e type);

/**
	Add property key and value for configuration.
	the data will be copied to config handle, you should free used data array.

	@param[in] 	config 		instance handler
	@param[in] 	attribute 				attribue key for value.
	@param[in] 	size	 				size of value
	@param[in] 	data	 				value array (binary type)

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_ALLOC_FAIL			allocation is failed
*/

net_nfc_error_e net_nfc_add_carrier_config_property (net_nfc_carrier_config_h config, uint16_t attribute, uint16_t size, uint8_t * data);

/**
	Remove the key and value from configuration, you can also remove the group  withe CREDENTIAL key.
	The the attribute is exist then it will be removed and also freed automatically.

	@param[in] 	config 		instance handler
	@param[in] 	attribute 				attribue key for value.

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_NO_DATA_FOUND		the given key is not found
	@exception NET_NFC_ALREADY_REGISTERED	the given attribute is already registered
*/

net_nfc_error_e net_nfc_remove_carrier_config_property (net_nfc_carrier_config_h config, uint16_t attribute);
/**
	Get the property value by attribute.

	@param[in] 	config 		instance handler
	@param[in] 	attribute 				attribue key for value.
	@param[out] 	size	 				size of value
	@param[out] 	data	 				value array (binary type)

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_NO_DATA_FOUND		The given key is not found
*/

net_nfc_error_e net_nfc_get_carrier_config_property (net_nfc_carrier_config_h config, uint16_t attribute, uint16_t * size, uint8_t ** data);

/**
	The group will be joined into the configure

	@param[in] 	config 		instance handler
	@param[in]	group				group handle

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_ALREADY_REGISTERED	The given group is already registered
*/

net_nfc_error_e net_nfc_append_carrier_config_group (net_nfc_carrier_config_h config, net_nfc_property_group_h group);

/**
	Remove the group from configure handle

	@param[in] 	config 		instance handler
	@param[in]	group				group handle

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_NO_DATA_FOUND		given handle pointer is not exist in the configure handle
*/

net_nfc_error_e net_nfc_remove_carrier_config_group (net_nfc_carrier_config_h config, net_nfc_property_group_h group);

/**
	Get the group from configure handle by index

	@param[in] 	config 		instance handler
	@param[in]	index				index number
	@param[out]	group				group handle

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_OUT_OF_BOUND			the index number is not bound of the max count
	@exception NET_NFC_NO_DATA_FOUND		this is should be happened if the configure handle is dammaged
*/

net_nfc_error_e net_nfc_get_carrier_config_group (net_nfc_carrier_config_h config, int index, net_nfc_property_group_h * group);

/**
	free the configure handle

	@param[in] 	config 		instance handler

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
*/

net_nfc_error_e net_nfc_free_carrier_config (net_nfc_carrier_config_h config);

/**
	create the group handle

	@param[out] 	group 		instance group handler

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_ALLOC_FAIL			allocation is failed
*/

net_nfc_error_e net_nfc_create_carrier_config_group (net_nfc_property_group_h * group, uint16_t attribute);

/**
	add property into the group

	@param[in] 	group 		instance group handler
	@param[in]	attribute		attribute of the property
	@param[in]	size			size of data (value)
	@param[in]	data			data of the property

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_ALLOC_FAIL			allocation is failed
	@exception NET_NFC_ALREADY_REGISTERED	the given key is already registered
*/

net_nfc_error_e net_nfc_add_carrier_config_group_property (net_nfc_property_group_h group, uint16_t attribute, uint16_t size, uint8_t * data);

/**
	get property from group handle

	@param[in] 	group 		instance group handler
	@param[in]	attribute		attribute of the property
	@param[out]	size			size of data (value)
	@param[out]	data			data of the property

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
  	@exception NET_NFC_NO_DATA_FOUND		the attribute does not exist in the group
*/

net_nfc_error_e net_nfc_get_carrier_config_group_property (net_nfc_property_group_h group, uint16_t attribute, uint16_t *size, uint8_t ** data);

/**
	remove the property from the group

	@param[in] 	group 		instance group handler
	@param[in]	attribute		attribute of the property

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
  	@exception NET_NFC_NO_DATA_FOUND		the attribute does not exist in the group
*/

net_nfc_error_e net_nfc_remove_carrier_config_group_property (net_nfc_property_group_h group, uint16_t attribute);


/**
	free the group

	@param[in] 	group 		instance group handler

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
*/

net_nfc_error_e net_nfc_free_carrier_group (net_nfc_property_group_h group);

/**
	Create record handler with config.

	@param[out] 	record 		record handler
	@param[in]	config	the wifi configure handle

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_ALLOC_FAIL			allocation is failed
*/

net_nfc_error_e net_nfc_create_ndef_record_with_carrier_config (ndef_record_h * record, net_nfc_carrier_config_h config);


/**
	create configure from the ndef record. the. the record must contained the configuration.
	config should be freed after using

	@param[in] 	record 		record handler
	@param[out]	config	the configure handle

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_ALLOC_FAIL			allocation is failed
	@exception NET_NFC_INVALID_FORMAT		if the record does not contained the valid format.
*/

net_nfc_error_e net_nfc_create_carrier_config_from_config_record (net_nfc_carrier_config_h* config, ndef_record_h record);

/**
	append record into the connection handover request for reponse message;

	@param[in] 	message 		ndef message handler
	@param[in] 	record 		record handler
	@param[in]	power_status	the power status of the current configuration

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_ALLOC_FAIL			allocation is failed
	@exception NET_NFC_INVALID_FORMAT		if the record does not contained the valid format.
	@exception NET_NFC_NO_DATA_FOUND		the attribute does not exist in the group
*/
net_nfc_error_e net_nfc_append_carrier_config_record (ndef_message_h message, ndef_record_h record, net_nfc_conn_handover_carrier_state_e power_status);

/**
	append record into the connection handover request for reponse message;

	@param[in] 	message 		ndef message handler
	@param[in] 	record 		record handler

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_ALLOC_FAIL			allocation is failed
	@exception NET_NFC_INVALID_FORMAT		if the record does not contained the valid format.
	@exception NET_NFC_NO_DATA_FOUND		Given record does not exist in the ndef message
*/

net_nfc_error_e net_nfc_remove_carrier_config_record (ndef_message_h message, ndef_record_h record);

/**
	get configure record from ndef message by index

	@param[in] 	message 		ndef message handler
	@param[in]	index
	@param[out] 	record 		record handler

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_ALLOC_FAIL			allocation is failed
	@exception NET_NFC_INVALID_FORMAT		if the record does not contained the valid format.
*/

net_nfc_error_e net_nfc_get_carrier_config_record (ndef_message_h message, int index, ndef_record_h * record);

/**
	get randome number from the connection request message

	@param[in] 	message 			ndef message handler
	@param[out] 	randome_number 	randome number

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_INVALID_FORMAT		if the record does not contained the valid format.
*/

net_nfc_error_e net_nfc_get_handover_random_number(ndef_message_h message, unsigned short* random_number);

/**
	get the count of the alternative (configuration) in the message

	@param[in] 	message 		ndef message handler
	@param[out] 	count	 	number configuration in the message

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_INVALID_FORMAT		if the record does not contained the valid format.
*/
net_nfc_error_e net_nfc_get_alternative_carrier_record_count (ndef_message_h message,  unsigned int * count);


/**
	get power status of the given configruation

	@param[in] 	message 		ndef message handler
	@param[in] 	index	 	index
	@param[out]	power_state	power state of the alternative

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_INVALID_FORMAT		if the record does not contained the valid format.
	@exception NET_NFC_NO_DATA_FOUND		there is no alternative record is found
	@exception NET_NFC_OUT_OF_BOUND			index number is out of message count
*/

net_nfc_error_e net_nfc_get_alternative_carrier_power_status (ndef_message_h message, int index, net_nfc_conn_handover_carrier_state_e * power_state);

/**
	set power status of the given configruation

	@param[in] 	message 		ndef message handler
	@param[in] 	index	 	index
	@param[in]	power_state	power state of the alternative

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_INVALID_FORMAT		if the record does not contained the valid format.
	@exception NET_NFC_NO_DATA_FOUND		there is no alternative record is found
	@exception NET_NFC_OUT_OF_BOUND			index number is out of message count
*/
net_nfc_error_e net_nfc_set_alternative_carrier_power_status (ndef_message_h message, int index, net_nfc_conn_handover_carrier_state_e power_status);

/**
	this function will get carrier type.

	@param[in] 	carrier_info 			connection handover carrier info handler
	@param[in] 	carrier_type 			record type. it can be a NET_NFC_CONN_HANDOVER_CARRIER_BT or NET_NFC_CONN_HANDOVER_CARRIER_WIFI or NET_NFC_CONN_HANDOVER_CARRIER_UNKNOWN.

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
*/

net_nfc_error_e net_nfc_get_alternative_carrier_type (ndef_message_h message, int index, net_nfc_conn_handover_carrier_type_e * power_state);


/**
	craete the connection handover request message

	@param[in] 	message 			connection handover carrier info handler

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_ALLOC_FAIL			allocation is failed
*/

net_nfc_error_e net_nfc_create_handover_request_message (ndef_message_h * message);

/**
	craete the connection handover select message

	@param[in] 	message 			connection handover carrier info handler

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_ALLOC_FAIL			allocation is failed
*/

net_nfc_error_e net_nfc_create_handover_select_message (ndef_message_h * message);

/**
	craete the connection handover error record message

	@param[out] 	record 			connection handover carrier info handler
	@param[in]	reason			error codes (reason)
	@param[in]	data				extra data for each error codes

	@return		return the result of the calling the function

	@exception NET_NFC_NULL_PARAMETER		parameter(s) has(have) illigal NULL pointer(s)
	@exception NET_NFC_ALLOC_FAIL			allocation is failed
*/
net_nfc_error_e net_nfc_create_handover_error_record (ndef_record_h * record, uint8_t reason, uint32_t data);


#ifdef __cplusplus
 }
#endif


#endif


